# -*- coding: utf-8 -*-

## Connect to server using socket from stdlib

import socket

# Socket server data:
gnc_socket = './gnc.socket'

# Connect to socket

sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

# Connect to socket

sock.connect(gnc_socket)

# Set timeout

timeout = 1
sock.settimeout(timeout)

# The client should read ever the socket

# While

# Read from input

# Send to socket

# Read from socket
sock.connect(gnc_socket)



import socket
from basic_queuetools.queue import read_queue_gen
from gnsocket.gn_socket import GNCSocket
# Standar lib
import asyncio
import functools
from multiprocessing import Manager, Queue, Lock

# contrib modules
import ujson as json

# Own module
from gnsocket.gn_socket import GNCSocket
from gnsocket.exceptions import clean_exception

# module tasktools
from tasktools.taskloop import coromask, renew, simple_fargs
from networktools.colorprint import gprint, bprint, rprint

from networktools.library import pattern_value, \
    fill_pattern, context_split, \
    gns_loads, gns_dumps
from networktools.library import my_random_string
from asyncio import shield

import ujson as json

tsleep = 2


class GNCSocketBase:

    def __init__(self, queue_n2t,
                 queue_t2n,
                 mode,
                 callback_exception=clean_exception,
                 *args,
                 **kwargs):
        self.qn2t = queue_n2t
        self.qt2n = queue_t2n
        self.address = kwargs.get('address', ('localhost', 6666))
        self.log_path = kwargs.get('log_path', '~/gnsocket_log')        
        self.mode = mode
        self.exception = callback_exception
        self.timeout = kwargs.get("timeout", 5)
        self.raise_timeout =  kwargs.get("raise_timeout", False)

    async def sock_write(self, gs, idc, *args, **kwargs):
        queue = self.qn2t
        await asyncio.sleep(.5)
        if idc in gs.clients:
            try:
                heartbeat = await gs.heart_beat(idc) 
                if heartbeat:
                    for msg in read_queue_gen(queue):
                        msg_send = json.dumps(msg)
                        idc_server = msg.get('idc_server')
                        send_msg = asyncio.wait_for(gs.send_msg(msg_send, idc), timeout=5)
                        await shield(send_msg)
                else:
                    print("Error en conexión socket sock_write")
                    print("heartbeat fail")
            except asyncio.TimeoutError as te:
                gs.logger.exception("Tiempo fuera en escritura %s, mode %s" %(
                    te, gs.mode))
                await asyncio.sleep(10)
            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                gs.logger.exception("Excepción por desconexión %s, mode %s"%(
                    conn_error,gs.mode))
                await asyncio.sleep(10)
                gs.set_status(False)
                del gs.clients[idc]
                if self.exception:
                    self.exception(ex, gs, idc)                
            except Exception as ex:
                gs.logger.exception("Error con modulo cliente gnsocket coro write %s" %ex)
                print("Error con modulo de escritura del socket IDC %s" % idc)
        else:
            await asyncio.sleep(20)
        return [gs, idc, *args], kwargs
    # socket communication terminal to engine

    async def sock_read(self, gs, idc, *args, **kwargs):
        queue = self.qt2n
        msg_from_engine = []
        await asyncio.sleep(.5)
        if idc in gs.clients:
            try:
                heartbeat = await gs.heart_beat(idc)
                recv_msg = gs.recv_msg(idc)
                datagram = await shield(recv_msg)
                if datagram not in {'', "<END>", 'null', None}:
                    msg_dict = json.loads(datagram)
                    msg = {'dt': msg_dict, 'idc': idc}
                    queue.put(msg)
            except asyncio.TimeoutError as te:
                gs.logger.exception("Tiempo fuera en escritura %s, mode %s" %(
                    te, gs.mode))
                await asyncio.sleep(10)
            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                gs.logger.exception("Excepción por desconexión %s, mode %s"%(
                    conn_error, gs.mode))
                await asyncio.sleep(10)
                gs.set_status(False)
                del gs.clients[idc]
                if self.exception:
                    self.exception(ex, gs, idc)                
            except Exception as ex:
                bprint("Excption on sock_read")
                rprint(ex)
                gs.logger.exception("Error con modulo cliente gnsocket coro read %s" %ex)           
                print("Some error %s en sock_read" % ex)
        else:
            await asyncio.sleep(20)
        return [gs, idc, args], kwargs

    def set_socket_task(self, callback_socket_task):
        self.socket_task = callback_socket_task



from gnsocket.gn_socket import GNCSocket
from gnsocket.conf.socket_conf import TEST_TEXT
import sys
import asyncio

from networktools.ssh import bridge, kill
from networktools.ports import used_ports, get_port
import socket

if __name__ == "__main__":
    mode = 'client'
    gs = GNCSocket(mode=mode)
    # only remote
    # up=used_ports()
    #(port, up)=get_port(up)
    # host_port=6677
    # host='10.54.217.15'
    # user='geodesia'
    #ssh_bridge=bridge(port, host_port, host, user)
    port = 6677
    host = socket.gethostbyname(socket.gethostname())
    address = (host, port)
    print(address)
    gs.set_address(address)
    loop = asyncio.get_event_loop()
    gs.set_loop(loop)
    # gs.accept()
    msg = TEST_TEXT

    idc = "X"

    # Testing message generator
    print("Status %s" % gs.status)
    for m in gs.generate_msg(msg):
        print(m.decode('utf-8'))
    # gs.send_msg(msg)
    # print(gs.conn)
    # Testing communicate
    # gs.accept()
    # print(gs.conn)

    async def send_msg(loop):
        try:
            idc = await gs.create_client()
        except Exception as x:
            print("Error al crear cliente: %s " % x)
            raise x
        while True:
            await asyncio.sleep(1)
            try:
                msg = ''
                while not msg == '<END>':
                    msg = await gs.recv_msg(idc)
                    if msg != '<END>' and msg is not None:
                        print(msg)
                    if msg == '<END>':
                        break

                this_msg = input("->")
                #
                if this_msg != "":
                    print("SEND :", this_msg)
                    await gs.send_msg(this_msg, idc)
                    await gs.send_msg('<END>', idc)

                    print("Ya enviado: " + this_msg)
                    if this_msg == "DONE":
                        print("Cerrando server")
            except KeyboardInterrupt as k:
                if gs.writer.can_write_eof():
                    gs.writer.write_eof()
                print("Cerrando con kb")
                gs.close()
                sys.exit()
            except Exception as exec:
                print("Error en conexion a puerto mediante bridge")
                # kill(ssh_bridge)
                raise exec
    try:
        loop.run_until_complete(send_msg(loop))
    except Exception as exec:
        print("Error %s " % exec)
        # kill(ssh_bridge)
    loop.close()
    print("-" * 20)
    print("Apagando")
    print("Listo!")



import socket
from basic_queuetools.queue import read_queue_gen
from gnsocket.gn_socket import GNCSocket
# Standar lib
import asyncio
import functools
from multiprocessing import Manager, Queue, Lock

# contrib modules
import ujson as json

# Own module
from gnsocket.gn_socket import GNCSocket
from gnsocket.socket_base import GNCSocketBase

# module tasktools
from tasktools.taskloop import coromask, renew, simple_fargs_out
from networktools.colorprint import gprint, bprint, rprint

from networktools.library import pattern_value, \
    fill_pattern, context_split, \
    gns_loads, gns_dumps
from networktools.library import my_random_string
from asyncio import shield, wait_for, wait

import ujson as json

tsleep = 2


class GNCSocketClient(GNCSocketBase):

    def __init__(self, queue_n2t, queue_t2n, *args, **kwargs):
        super().__init__(queue_n2t, queue_t2n, 'client', *args, **kwargs)
        self.set_socket_task(self.socket_task)

    def socket_task(self):
        # client socket
        loop = asyncio.get_event_loop()
        self.connected = False
        with GNCSocket(mode='client', timeout=self.timeout,
                       raise_timeout=self.raise_timeout,
                       log_path=self.log_path) as gs:
            try:
                self.loop = loop
                gs.set_address(self.address)
                gs.set_loop(loop)

                async def socket_io():
                    idc = await gs.create_client()
                    try:
                        args = [gs, idc]
                        task_1 = loop.create_task(
                            coromask(
                                self.sock_read,
                                args, {},
                                simple_fargs_out)
                        )
                        task_1.add_done_callback(
                            functools.partial(
                                renew,
                                task_1,
                                self.sock_read,
                                simple_fargs_out)
                        )
                        args = [gs, idc]
                        # task write
                        task_2 = loop.create_task(
                            coromask(
                                self.sock_write,
                                args, {},
                                simple_fargs_out)
                        )
                        task_2.add_done_callback(
                            functools.partial(
                                renew,
                                task_2,
                                self.sock_write,
                                simple_fargs_out)
                        )
                    except Exception as ex:
                        gs.abort(idc)
                        task_1.cancel()
                        task_2.cancel()
                        raise ex
            except Exception as ex:
                gs.logger.exception("Error con modulo cliente gnsocket %s" %ex)                
                print("Eception as %s" %ex)
                raise ex

                ########
                # Insert a coroutine with reader and writer tasks

            async def activate_sock():
                try:
                    await shield(wait_for(socket_io(), timeout=10))
                    return "socket loaded"
                except asyncio.TimeoutError as te:
                    gs.logger.exception("Tiempo fuera en escritura %s, mode %s" %(
                        te, gs.mode))
                    await asyncio.sleep(10)
                except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                    gs.logger.exception("Excepción por desconexión %s, mode %s"%(
                        conn_error,gs.mode))
                    await asyncio.sleep(10)                                                           
            future1 = loop.create_task(activate_sock())
            if not loop.is_running():
                loop.run_forever()


if __name__ == "__main__":
    address = (socket.gethostbyname(socket.gethostname()), 5500)
    client = GNCSocketClient(address=address)
    client.socket_task()



import asyncio
from tasktools.taskloop import coromask, renew, simple_fargs
import functools
from termcolor import colored, cprint

"""
A test example to raise an exceptio throw an asyncio task
"""

def gprint(text):
    msg = colored(text, 'green', attrs=['reverse', 'blink'])
    print(msg)

def bprint(text):
    msg = colored(text, 'blue', attrs=['reverse', 'blink'])
    print(msg)

def rprint(text):
    msg = colored(text, 'red', attrs=['reverse', 'blink'])
    print(msg)

if __name__ == "__main__":
    mode = 'server'
    print("Entrando a loop")
    loop = asyncio.get_event_loop()

    async def div0(n):
        try:
            print(n)
            n/0
        except:
            raise Exception()

    def div0_task():
        try:
            args=[10]
            task=loop.create_task(
                coromask(
                    div0,
                    args,
                    simple_fargs)
            )
            task.add_done_callback(
                functools.partial(
                    renew,
                    task,
                    div0,
                    simple_fargs)
            )
        except Exception as exec:
            raise exec

    try:
        div0_task()
        loop.run_forever()
    except:
        raise Exception()
    loop.close()



from setuptools import setup, find_packages

setup(name='gnsocket',
      version='1.2.2',
      description='GPS Network Socket, with asyncio stream manager',
      url='https://gitlab.com/pineiden/gus',
      author='David Pineda Osorio',
      author_email='dpineda@csn.uchile.cl',
      license='GPL3',
      install_requires=["networktools","basic_queuetools", "basic_logtools",'chardet'],
      packages=find_packages(),      
      include_package_data=True,      
      package_dir={'gnsocket': 'gnsocket'},      
      zip_safe=False)



# Standar lib
import asyncio
import functools
from multiprocessing import Manager, Queue, Lock

# contrib modules
import ujson as json

# Own module
from gnsocket.gn_socket import GNCSocket
from gnsocket.socket_base import GNCSocketBase

# module tasktools
from tasktools.taskloop import coromask, renew, simple_fargs_out
from networktools.colorprint import gprint, bprint, rprint

from networktools.library import pattern_value, \
    fill_pattern, context_split, \
    gns_loads, gns_dumps

from basic_queuetools.queue import read_queue_gen, send_queue

tsleep = 2
from asyncio import shield, wait_for, wait


class GNCSocketServer(GNCSocketBase):

    def __init__(self, queue_n2t, queue_t2n, *args, **kwargs):
        super().__init__(queue_n2t, queue_t2n, 'server', *args, **kwargs)
        self.set_socket_task(self.socket_task)

    def socket_task(self):
        with GNCSocket(mode=self.mode, timeout=self.timeout, raise_timeout=self.raise_timeout, log_path=self.log_path) as gs:
            loop = asyncio.get_event_loop()
            self.loop = loop
            gs.set_address(self.address)
            gs.set_loop(loop)
            try:
                async def socket_io(reader, writer):
                    try:
                        idc = await gs.set_reader_writer(reader, writer)
                        # First time welcome
                        welcome = json.dumps(
                            {"msg": "Welcome to socket", 'idc_server': idc})
                        await wait_for(gs.send_msg(welcome, idc), timeout=10)
                        await asyncio.sleep(0.1)
                        # task reade
                    except asyncio.TimeoutError as te:
                        gs.logger.exception("Tiempo fuera en escritura %s, mode %s" %(
                            te, gs.mode))
                        await asyncio.sleep(10)
                    except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                        gs.logger.exception("Excepción por desconexión %s, mode %s"%(
                            conn_error, gs.mode))
                        await asyncio.sleep(10)                                                                                               
                    try:
                        args = [gs, idc]
                        task_1 = loop.create_task(
                            coromask(
                                self.sock_read,
                                args, {},
                                simple_fargs_out)
                        )
                        task_1.add_done_callback(
                            functools.partial(
                                renew,
                                task_1,
                                self.sock_read,
                                simple_fargs_out)
                        )
                        args = [gs, idc]
                        # task write
                        task_2 = loop.create_task(
                            coromask(
                                self.sock_write,
                                args, {},
                                simple_fargs_out)
                        )
                        task_2.add_done_callback(
                            functools.partial(
                                renew,
                                task_2,
                                self.sock_write,
                                simple_fargs_out)
                        )
                    except Exception as exe:
                        raise exe
                future = loop.create_task(
                    gs.create_server(socket_io, loop))
                if not loop.is_running():
                    loop.run_forever()
                else:
                    loop.run_until_complete(future)
            except KeyboardInterrupt as k:
                print("Closing with keyboard")
                gs.logger.exception("Cierre forzado desde teclado %s" %ex)                
                loop.run_until_complete(gs.wait_closed())
                raise k
            except Exception as ex:
                gs.logger.exception("Error con modulo cliente gnsocket %s" %ex)                                
                print("Otra exception", ex)

        def close(self):
            self.gs.close()
            loop = asyncio.get_event_loop()
            loop.close()
